#include "generator.h"
#include "SDL.h"
#include "SDL_ttf.h"
#include "ui.h"
#include "ui-sdl.h"

extern char rom_fn[257];
extern SDL_Surface * screen;
extern TTF_Font * font;

/* This is taken from libSGE -- http://freshmeat.net/projects/sge/ */
// void sge_FilledRectAlpha(SDL_Surface *surface, Sint16 x1, Sint16 y1, Sint16 x2, Sint16 y2, Uint32 color, Uint8 alpha)
void RectAlpha(SDL_Surface * surface, SDL_Rect * rect, Uint32 color, Uint8 alpha)
{
	Sint16 x1 = rect->x;
	Sint16 y1 = rect->y;
	Sint16 x2 = rect->x + rect->w;
	Sint16 y2 = rect->y + rect->h;
	
	/* Fix coords */
	Sint16 tmp;
	if(x1>x2){
		tmp=x1; x1=x2; x2=tmp;
	}
	if(y1>y2){
		tmp=y1; y1=y2; y2=tmp;
	}
	
	Uint32 Rmask = surface->format->Rmask, Gmask = surface->format->Gmask, Bmask = surface->format->Bmask, Amask = surface->format->Amask;
	Uint32 R,G,B,A=0;
	Sint16 x,y;
	
	if (SDL_MUSTLOCK(surface))
		if (SDL_LockSurface(surface) < 0)
			return;
	
	switch (surface->format->BytesPerPixel) {
		case 1: { /* Assuming 8-bpp */
			Uint8 *row, *pixel;
			Uint8 dR, dG, dB;
			
			Uint8 sR = surface->format->palette->colors[color].r;
			Uint8 sG = surface->format->palette->colors[color].g;
			Uint8 sB = surface->format->palette->colors[color].b;
			
			for(y = y1; y<=y2; y++){
				row = (Uint8 *)surface->pixels + y*surface->pitch;
				for(x = x1; x <= x2; x++){
					pixel = row + x;
					
					dR = surface->format->palette->colors[*pixel].r;
					dG = surface->format->palette->colors[*pixel].g;
					dB = surface->format->palette->colors[*pixel].b;
					
					dR = dR + ((sR-dR)*alpha >> 8);
					dG = dG + ((sG-dG)*alpha >> 8);
					dB = dB + ((sB-dB)*alpha >> 8);
				
					*pixel = SDL_MapRGB(surface->format, dR, dG, dB);
				}
			}
		}
		break;

		case 2: { /* Probably 15-bpp or 16-bpp */
			Uint16 *row, *pixel;
			Uint32 dR=(color & Rmask),dG=(color & Gmask),dB=(color & Bmask),dA=(color & Amask);
			
			for(y = y1; y<=y2; y++){
				row = (Uint16 *)surface->pixels + y*surface->pitch/2;
				for(x = x1; x <= x2; x++){
					pixel = row + x;

					R = ((*pixel & Rmask) + (( dR - (*pixel & Rmask) ) * alpha >> 8)) & Rmask;
					G = ((*pixel & Gmask) + (( dG - (*pixel & Gmask) ) * alpha >> 8)) & Gmask;
					B = ((*pixel & Bmask) + (( dB - (*pixel & Bmask) ) * alpha >> 8)) & Bmask;
					if( Amask )
						A = ((*pixel & Amask) + (( dA - (*pixel & Amask) ) * alpha >> 8)) & Amask;

					*pixel= R | G | B | A;
				}
			}
		}
		break;

		case 3: { /* Slow 24-bpp mode, usually not used */
			Uint8 *row,*pix;
			Uint8 dR, dG, dB, dA;
  			Uint8 rshift8=surface->format->Rshift/8; 
			Uint8 gshift8=surface->format->Gshift/8; 
			Uint8 bshift8=surface->format->Bshift/8;
			Uint8 ashift8=surface->format->Ashift/8;
			
			Uint8 sR = (color>>surface->format->Rshift)&0xff;
			Uint8 sG = (color>>surface->format->Gshift)&0xff;
			Uint8 sB = (color>>surface->format->Bshift)&0xff;
			Uint8 sA = (color>>surface->format->Ashift)&0xff;
				
			for(y = y1; y<=y2; y++){
				row = (Uint8 *)surface->pixels + y * surface->pitch;
				for(x = x1; x <= x2; x++){
					pix = row + x*3;

					dR = *((pix)+rshift8); 
            		dG = *((pix)+gshift8);
            		dB = *((pix)+bshift8);
					dA = *((pix)+ashift8);
					
					dR = dR + ((sR-dR)*alpha >> 8);
					dG = dG + ((sG-dG)*alpha >> 8);
					dB = dB + ((sB-dB)*alpha >> 8);
					dA = dA + ((sA-dA)*alpha >> 8);

					*((pix)+rshift8) = dR; 
            		*((pix)+gshift8) = dG;
            		*((pix)+bshift8) = dB;
					*((pix)+ashift8) = dA;
				}
			}
					
		}
		break;

		case 4: { /* Probably 32-bpp */
			Uint32 *row, *pixel;
			Uint32 dR=(color & Rmask),dG=(color & Gmask),dB=(color & Bmask),dA=(color & Amask);
		
			for(y = y1; y<=y2; y++){
				row = (Uint32 *)surface->pixels + y*surface->pitch/4;
				for(x = x1; x <= x2; x++){
					pixel = row + x;

					R = ((*pixel & Rmask) + (( dR - (*pixel & Rmask) ) * alpha >> 8)) & Rmask;
					G = ((*pixel & Gmask) + (( dG - (*pixel & Gmask) ) * alpha >> 8)) & Gmask;
					B = ((*pixel & Bmask) + (( dB - (*pixel & Bmask) ) * alpha >> 8)) & Bmask;
					if( Amask )
						A = ((*pixel & Amask) + (( dA - (*pixel & Amask) ) * alpha >> 8)) & Amask;

					*pixel= R | G | B | A;
				}
			}
		}
		break;
	}
	
	if (SDL_MUSTLOCK(surface)) {
		SDL_UnlockSurface(surface);
	}
}

/* Some of this is based on DCGenerator's rom select screen */
void uiext_dirlist()	{
	char fn[1025][257];
	char realfn[300];
	int count = 0;
	int romcount = 0;
	dirent_t *ent;
	uint32 hnd;
	int sel = 0, scsel = 0, prevsel = 0, offset = 0, done = 0, redraw = 1;
	SDL_Surface * temp;
	SDL_Rect rect;
	SDL_Color color = {0xFF, 0xFF, 0xFF};
	SDL_Event event;
	
	hnd = fs_open("/cd/roms", O_RDONLY | O_DIR);
	
	ent = fs_readdir(hnd);
	count = 0;    
	while (ent && count < 1025)	{
		if(ent->size != -1)	{
			char *extension = &ent->name[0] + strlen(ent->name) - 3;
			if(stricmp(extension, "smd") && stricmp(extension, "bin"))	{
				ent = fs_readdir(hnd);
				continue;
			}
		}
		strcpy(fn[count], ent->name);
		count++;
		romcount++;
		ent = fs_readdir(hnd);
	}
	
	while(done == 0)	{
		SDL_PollEvent(&event);
		switch(event.type)	{
			case SDL_JOYBUTTONDOWN:
				if(event.jbutton.button == 0)	{
					sprintf(realfn, "/cd/roms/%s", fn[sel]);
					strcpy(rom_fn, realfn);
					done = 1;
				}
				break;
			case SDL_JOYHATMOTION:
				if(event.jhat.value == SDL_HAT_UP && sel != 0)	{
					scsel--;
					sel--;
					redraw = 1;
				}
				if(event.jhat.value == SDL_HAT_DOWN && sel < romcount)	{
					scsel++;
					sel++;
					redraw = 1;
				}
				break;
		}
		if(redraw == 1)	{
			redraw = 0;
			if(sel <= romcount)	{
				rect.x = 0;
				rect.y = 0;
				rect.w = 640;
				rect.h = 480;
				SDL_FillRect(screen, &rect, SDL_MapRGB(screen->format, 0, 0, 0));
				temp = TTF_RenderText_Solid(font, "Select a ROM:", color);
				rect.x = 32;
				rect.y = 32;
				rect.w = 192;
				rect.h = 20;
				SDL_BlitSurface(temp, NULL, screen, &rect);
				if(scsel > 19)	{
					offset++;
					scsel = 19;
				}
				else if(scsel < 0)	{
					offset--;
					scsel = 0;
				}
					
				for(count = 0; count < 20; count++)	{
					if(fn[count + offset][0] != '\0')	{
						temp = TTF_RenderText_Solid(font, fn[count + offset], color);
						rect.x = 64;
						rect.y = 64 + (count * 20);
						rect.w = 512;
						rect.h = 20;
						SDL_BlitSurface(temp, NULL, screen, &rect);
						SDL_FreeSurface(temp);
					}
				}
				rect.x = 64;
				rect.y = 64 + (scsel * 20);
				rect.w = 512;
				rect.h = 20;
				RectAlpha(screen, &rect, SDL_MapRGB(screen->format, 0, 0, 0xFF), 127);
				SDL_Flip(screen);
			}
			prevsel = sel;
		}
		else	{
			sel = prevsel;
			scsel = sel - offset;
		}
	}
}
