/* Generator is (c) James Ponder, 1997-2001 http://www.squish.net/generator/ */

/* Generator-SDL is (c) 2003 Lawrence Sebald */
/* user interface for SDL (15/16 bit)
   based on the user interface for allegro (uip-allegro.c) */

#include <stdio.h>

#include "generator.h"
#include "SDL.h"
#include "SDL_ttf.h"

#include "uip.h"
#include "ui.h"
#include "ui-sdl.h"
#include "vdp.h"
#include "cpu68k.h"
#include "mem68k.h"

/*** forward reference declarations ***/
#ifdef _arch_dreamcast
void uiext_dirlist();
#endif

/*** static variables ***/

static uint8 uip_vga = 0;       /* flag for whether in VGA more or not */
static uint8 uip_displaybanknum = 0;    /* view this one, write to other one */
static SDL_Surface * uip_bank0;
static SDL_Surface * uip_bank1;
static uint8 *uip_writebank;
static uint8 uip_keypoll = 0;   /* flag indicating requires polling */
static t_uipinfo *uip_uipinfo = NULL;   /* uipinfo */
static int uip_forceredshift = -1;      /* if set, forces red shift position */
static int uip_forcegreenshift = -1;    /* if set, forces green shift position */
static int uip_forceblueshift = -1;     /* if set, forces blue shift position */
static SDL_Joystick * cont1;
TTF_Font * font;
SDL_Surface * screen;
static int SDLinitted = 0;
extern uint8 ui_frameskip;
static time_t timesince;


/*** Code ***/
int uip_init(t_uipinfo * uipinfo)	{
	uip_uipinfo = uipinfo;
	SDL_Surface * string;
	SDL_Event event;
	SDL_Color color = {0xFF, 0xFF, 0xFF};
	SDL_Rect rect;
	int ddone = 0;

	if(SDLinitted == 0)	{
		if(SDL_Init(SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_JOYSTICK) < 0)	{
			return -1;
		}
		SDL_ShowCursor(SDL_DISABLE);
		SDL_WM_SetCaption("Generator-SDL", NULL);
		screen = SDL_SetVideoMode(640, 480, 16, SDL_SWSURFACE);
		SDL_JoystickEventState(SDL_ENABLE);
		cont1 = SDL_JoystickOpen(0);
		if(TTF_Init())	{
			return -1;
		}
		#ifndef _arch_dreamcast
		font = TTF_OpenFont("font.ttf", 16);
		#else
		font = TTF_OpenFont("/cd/fonts/font.ttf", 16);
		SDL_PollEvent(&event);
		string = TTF_RenderText_Solid(font, "Disclaimer:", color);
		rect.x = 32;
		rect.y = 64;
		rect.w = 576;
		rect.h = 20;
		SDL_BlitSurface(string, NULL, screen, &rect);
		string = TTF_RenderText_Solid(font, "This is a test version! It is not perfect!", color);
		rect.y = 128;
		SDL_BlitSurface(string, NULL, screen, &rect);
		string = TTF_RenderText_Solid(font, "Feel free to report bugs to me, but do not", color);
		rect.y += 20;
		SDL_BlitSurface(string, NULL, screen, &rect);
		string = TTF_RenderText_Solid(font, "ask questions like \"When is the next release?\",", color);
		rect.y += 20;
		SDL_BlitSurface(string, NULL, screen, &rect);
		string = TTF_RenderText_Solid(font, "\"Why isn\'t [insert feature here] supported?\",", color);
		rect.y += 20;
		SDL_BlitSurface(string, NULL, screen, &rect);
		string = TTF_RenderText_Solid(font, "or \"Why doesn\'t [insert game here] work?\".", color);
		rect.y += 20;
		SDL_BlitSurface(string, NULL, screen, &rect);
		string = TTF_RenderText_Solid(font, "Also, do not tell me \"Your emu sux!\" or any", color);
		rect.y += 20;
		SDL_BlitSurface(string, NULL, screen, &rect);
		string = TTF_RenderText_Solid(font, "thing like it!", color);
		rect.y += 20;
		SDL_BlitSurface(string, NULL, screen, &rect);
		string = TTF_RenderText_Solid(font, "If you do, I will be very upset, and it isn't", color);
		rect.y += 20;
		SDL_BlitSurface(string, NULL, screen, &rect);
		string = TTF_RenderText_Solid(font, "a very good idea to upset a programmer! :)", color);
		rect.y += 20;
		SDL_BlitSurface(string, NULL, screen, &rect);
		string = TTF_RenderText_Solid(font, "That being said, enjoy ljsdcgen Version 1!", color);
		rect.y += 20;
		SDL_BlitSurface(string, NULL, screen, &rect);
		string = TTF_RenderText_Solid(font, "Press Start to continue.", color);
		rect.y += 20;
		SDL_BlitSurface(string, NULL, screen, &rect);
		SDL_Flip(screen);
		while(ddone != 1)	{
			SDL_PollEvent(&event);
			switch(event.type)	{
				case SDL_JOYBUTTONDOWN:
					if(event.jbutton.button == 4)
						ddone = 1;
						SDL_FreeSurface(string);
					break;
			}
		}
		#endif
		SDLinitted = 1;
	}
	return 0;
}

int uip_initjoysticks(void)	{
	/* This is taken care of in the main init */
	return 0;
}

int uip_vgamode(void)	{
	uip_vga = 1;
	if (uip_forceredshift != -1 && uip_forcegreenshift != -1 &&
		uip_forceblueshift != -1)	{
		uip_uipinfo->redshift = uip_forceredshift;
		uip_uipinfo->greenshift = uip_forcegreenshift;
		uip_uipinfo->blueshift = uip_forceblueshift;
	} 
	else {
	uip_uipinfo->redshift = ui_topbit(SDL_MapRGB(screen->format, 255, 0, 0)) - 4;
	uip_uipinfo->greenshift = ui_topbit(SDL_MapRGB(screen->format, 0, 255, 0)) - 4;
	uip_uipinfo->blueshift = ui_topbit(SDL_MapRGB(screen->format, 0, 0, 255)) - 4;
	}
	if(SDLinitted == 1)	{
		uip_bank0 = SDL_AllocSurface(SDL_SWSURFACE, 640, 480, 16, 0, 0, 0, 0);
		uip_bank1 = SDL_AllocSurface(SDL_SWSURFACE, 640, 480, 16, 0, 0, 0, 0);
		SDLinitted = 2;
	}
	uip_uipinfo->screenmem0 = uip_bank0->pixels;
	uip_uipinfo->screenmem1 = uip_bank1->pixels;
	uip_uipinfo->linewidth = 2 * 640;       /* 16 bit */
	uip_displaybank(0);           /* set current to 0th bank */
	uip_clearscreen();            /* clear bank */
	ui_setupscreen();             /* setup bank */
	uip_displaybank(-1);          /* toggle bank */
	uip_clearscreen();            /* clear bank */
	ui_setupscreen();             /* setup bank */
	return 0;
}

int uip_setcolourbits(int red, int green, int blue)	{
	uip_forceredshift = red;
	uip_forcegreenshift = green;
	uip_forceblueshift = blue;
	return 0;
}

void uip_displaybank(int bank)	{
	if (bank == -1)
		bank = uip_displaybanknum ^ 1;
	uip_writebank = bank ? uip_bank0->pixels : uip_bank1->pixels;
	SDL_BlitSurface(bank ? uip_bank1 : uip_bank0, NULL, screen, NULL);
	SDL_UpdateRect(screen, 0, 0, 0, 0);
	uip_uipinfo->screenmem_w = bank ? uip_uipinfo->screenmem0 : uip_uipinfo->screenmem1;
	uip_displaybanknum = bank;
}

void uip_clearscreen(void)	{
	int y;
	for (y = 0; y < 480; y++) {
		memset(uip_uipinfo->screenmem_w + uip_uipinfo->linewidth * y, 0, 640 * 2);
	}
}

void uip_clearmiddle(void)	{
	int i;
	for (i = 0; i < 240; i++) {
		memset(uip_uipinfo->screenmem_w +
		(uip_uipinfo->linewidth) * (120 + i) + 160 * 2, 0, 2 * 320);
	}
}

/* uip_singlebank sets write address to the current display screen */

void uip_singlebank(void)	{
	uip_uipinfo->screenmem_w = uip_displaybanknum ? uip_uipinfo->screenmem1 :
		uip_uipinfo->screenmem0;
}

/* uip_doublebank sets write address to hidden display screen */

void uip_doublebank(void)	{
	uip_uipinfo->screenmem_w = uip_displaybanknum ? uip_uipinfo->screenmem0 :
		uip_uipinfo->screenmem1;
}

void uip_textmode(void)	{
  SDL_Quit();
}

void uip_cleanup()	{
	SDL_FreeSurface(screen);
	SDL_FreeSurface(uip_bank0);
	SDL_FreeSurface(uip_bank1);
}

int uip_checkkeyboard(void)	{
	SDL_Event event;
	int foo = 0;
		
	SDL_PollEvent(&event);
	switch(event.type)	{
		case SDL_JOYAXISMOTION:
			if(event.jaxis.axis == 0 && event.jaxis.value > 100)	{
				foo = 1;
			}
			if(event.jaxis.axis == 1 && event.jaxis.value > 64 && ui_frameskip > 1 && time(NULL) - timesince > 1)	{
				ui_frameskip--;
				timesince = time(NULL);
			}
			if(event.jaxis.axis == 1 && event.jaxis.value < -64 && ui_frameskip < 10 && time(NULL) - timesince > 1)	{
				ui_frameskip++;
				timesince = time(NULL);
			}
			break;

		case SDL_JOYBUTTONDOWN:
			if(event.jbutton.button == 2)
				mem68k_cont[0].a = 1;
			else if(event.jbutton.button == 3)
				mem68k_cont[0].b = 1;
			else if(event.jbutton.button == 1)
				mem68k_cont[0].c = 1;
			else if(event.jbutton.button == 4)
				mem68k_cont[0].start = 1;
			break;
			
		case SDL_JOYBUTTONUP:
			if(event.jbutton.button == 2)
				mem68k_cont[0].a = 0;
			else if(event.jbutton.button == 0)
				mem68k_cont[0].b = 0;
			else if(event.jbutton.button == 1)
				mem68k_cont[0].c = 0;
			else if(event.jbutton.button == 4)
				mem68k_cont[0].start = 0;
			break;
			
		case SDL_JOYHATMOTION:
			if(event.jhat.value == SDL_HAT_UP)	{
				mem68k_cont[0].up = 1;
				mem68k_cont[0].down = 0;
				mem68k_cont[0].left = 0;
				mem68k_cont[0].right = 0;
			}
			else if(event.jhat.value == SDL_HAT_DOWN)	{
				mem68k_cont[0].up = 0;
				mem68k_cont[0].down = 1;
				mem68k_cont[0].left = 0;
				mem68k_cont[0].right = 0;
			}
			else if(event.jhat.value == SDL_HAT_RIGHT)	{
				mem68k_cont[0].up = 0;
				mem68k_cont[0].down = 0;
				mem68k_cont[0].left = 0;
				mem68k_cont[0].right = 1;
			}
			else if(event.jhat.value == SDL_HAT_LEFT)	{
				mem68k_cont[0].up = 0;
				mem68k_cont[0].down = 0;
				mem68k_cont[0].left = 1;
				mem68k_cont[0].right = 0;
			}
			else if(event.jhat.value == SDL_HAT_CENTERED)	{
				mem68k_cont[0].up = 0;
				mem68k_cont[0].down = 0;
				mem68k_cont[0].left = 0;
				mem68k_cont[0].right = 0;
			}
			break;
		case SDL_KEYDOWN:
			switch(event.key.keysym.sym)	{
				case SDLK_F1:
					ui_fkeys |= 1 << 1;
					break;
				case SDLK_F2:
					ui_fkeys |= 1 << 2;
					break;
				case SDLK_F3:
					ui_fkeys |= 1 << 3;
					break;
				case SDLK_F4:
					ui_fkeys |= 1 << 4;
					break;
				case SDLK_F5:
					ui_fkeys |= 1 << 5;
					break;
				case SDLK_F6:
					ui_fkeys |= 1 << 6;
					break;
				case SDLK_F7:
					ui_fkeys |= 1 << 7;
					break;
				case SDLK_F8:
					ui_fkeys |= 1 << 8;
					break;
				case SDLK_F9:
					ui_fkeys |= 1 << 9;
					break;
				case SDLK_F10:
					ui_fkeys |= 1 << 10;
					break;
				case SDLK_F11:
					ui_fkeys |= 1 << 11;
					break;
				case SDLK_F12:
					ui_fkeys |= 1 << 12;
					break;
			}
			break;
	}
	return foo;
}

unsigned int uip_whichbank(void)	{
	/* returns 0 or 1 - the bank being VIEWED */
	return uip_displaybanknum;
}

/* What does this do? */
void uip_vsync(void)	{	}

uint8 uip_getchar(void)	{
	char c;
	return c;
}

#ifdef _arch_dreamcast
char * strerror(int errnum)	{	return "ERROR!!!";	}
#endif
